@@ -1,3 +1,48 @@
+#' Set and query GDAL configuration options
+#'
+#' These functions can get and set configuration options for GDAL, for fine
+#' control over specific GDAL behaviours. 
+#' 
+#' Configuration options may also be set as environment variables. 
+#' 
+#' See [GDAL config options](https://trac.osgeo.org/gdal/wiki/ConfigOptions) for
+#' details on available options. 
+#' 
+#' @param option GDAL config name (see Details), character string
+#' @param value value for config option, character string
+#'
+#' @return character string for `vapour_get_config`, integer 1 for successful `vapour_set_config()`
+#' @export
+#'
+#' @examples
+#' \dontrun{
+#' (orig <- vapour_get_config("GDAL_CACHEMAX"))
+#' vapour_set_config("GDAL_CACHEMAX", "64")
+#' vapour_get_config("GDAL_CACHEMAX")
+#' vapour_set_config("GDAL_CACHEMAX", orig)
+#' }
+vapour_set_config <- function(option, value) {
+  option <- as.character(option[1L])
+  value <- as.character(value[1])
+  if (length(option) < 1 || nchar(option) < 1 || is.na(option) || is.null(option) ) {
+    stop(sprintf("invalid 'option': %s - must be valid character string"))
+  }
+  if (length(value) < 1 || nchar(value) < 1 || is.na(value) || is.null(value) ) {
+    stop(sprintf("invalid 'value': %s - must be valid character string"))
+  }
+  set_gdal_config_cpp(option, value)
+}
+
+#' @export
+#' @name vapour_set_config
+vapour_get_config  <- function(option) {
+  if (length(option) < 1 || nchar(option) < 1 || is.na(option) || is.null(option) ) {
+    stop(sprintf("invalid 'option': %s - must be valid character string"))
+  }
+  get_gdal_config_cpp(option)
+}
+
+
 #' PROJ4 string to WKT
 #'
 #' Convert a projstring to Well Known Text.
@@ -8,12 +53,18 @@
 #' Note that no sanitizing is done on inputs, we literally just 'OGRSpatialReference.SetFromUserInput(crs)' and
 #' give the output as WKT. If it's an error in GDAL it's an error in R.
 #'
-#' You can get some funky outputs from random strings, so don't do that. Common sensible inputs are WKT variants,
+#' Common inputs are WKT variants,
 #' 'AUTH:CODE's e.g. 'EPSG:3031', the 'OGC:CRS84' for long,lat WGS84, 'ESRI:<code>' and other authority variants, and
 #' datum names such as 'WGS84','NAD27' recognized by PROJ itself.
 #'
-#' See help for 'SetFromUserInput' in 'OGRSpatialReference', and 'proj_create_crs_to_crs' in PROJ.
-#'
+#' See help for 'SetFromUserInput' in 'OGRSpatialReference', and 'proj_create_crs_to_crs'.
+#' 
+#' [c.proj_create_crs_to_crs](https://proj.org/development/reference/functions.html#c.proj_create_crs_to_crs) 
+#' 
+#' [c.proj_create](https://proj.org/development/reference/functions.html#c.proj_create)
+#' 
+#' [SetFromUserInput](https://gdal.org/doxygen/classOGRSpatialReference.html#aec3c6a49533fe457ddc763d699ff8796)
+#' 
 #' @param crs projection string, see Details.
 #' @export
 #' @return WKT2 projection string
@@ -117,8 +168,7 @@ vapour_all_drivers <- function() {
 #' @export
 #' @param dsource data source string (i.e. file name or URL or database connection string)
 vapour_driver <- function(dsource) {
-  if (!is.character(dsource)) stop("'dsource' must be a character vector")
-  if (!nchar(dsource) > 0) stop("'dsource' is an empty string")
+  dsource <- .check_dsn_single(dsource)
   driver_id_gdal_cpp(dsource);
 }
 
