@@ -65,11 +65,10 @@ inline CharacterVector gdal_layer_geometry_name(OGRLayer *poLayer) {
 inline NumericVector gdal_layer_extent(OGRLayer *poLayer) {
   
   OGREnvelope poEnvelope;
-  double err = poLayer ->GetExtent(&poEnvelope,true);
-  if (err) {
-    // do nothing
-  } else {
-    // more nothin
+  OGRErr err; 
+  err = poLayer ->GetExtent(&poEnvelope,true);
+  if (err != CE_None) {
+    Rprintf("problem in get extent\n");
   }
   NumericVector out(4); 
   out[0] = poEnvelope.MinX;
@@ -81,11 +80,11 @@ inline NumericVector gdal_layer_extent(OGRLayer *poLayer) {
 
 // this force function takes cheap count, checks, then more expensive, checks,
 // then iterates and resets reading
-inline double force_layer_feature_count(OGRLayer *poLayer) {
-  double out;
-  out = (double)poLayer->GetFeatureCount(false);
+inline R_xlen_t force_layer_feature_count(OGRLayer *poLayer) {
+  R_xlen_t out;
+  out = poLayer->GetFeatureCount(false);
   if (out == -1) {
-    out = (double)poLayer->GetFeatureCount(true);
+    out = poLayer->GetFeatureCount(true);
   }
   if (out == -1) {
     out = 0;
@@ -167,13 +166,13 @@ inline List gdal_list_drivers()
   Rcpp::LogicalVector isvirt(n);
   for (int idriver = 0; idriver < n; idriver++) {
     GDALDriver *dr = GetGDALDriverManager()->GetDriver(idriver);
-    sname(idriver) = GDALGetDriverShortName(dr);
-    lname(idriver) = GDALGetDriverLongName(dr);
-    isvector(idriver) = (dr->GetMetadataItem(GDAL_DCAP_VECTOR) != NULL);
-    israster(idriver) = (dr->GetMetadataItem(GDAL_DCAP_RASTER) != NULL);
-    iscopy(idriver) = (dr->GetMetadataItem(GDAL_DCAP_CREATECOPY) != NULL);
-    iscreate(idriver) = (dr->GetMetadataItem(GDAL_DCAP_CREATE) != NULL);
-    isvirt(idriver) = (dr->GetMetadataItem(GDAL_DCAP_VIRTUALIO) != NULL);
+    sname[idriver] = GDALGetDriverShortName(dr);
+    lname[idriver] = GDALGetDriverLongName(dr);
+    isvector[idriver] = (dr->GetMetadataItem(GDAL_DCAP_VECTOR) != NULL);
+    israster[idriver] = (dr->GetMetadataItem(GDAL_DCAP_RASTER) != NULL);
+    iscopy[idriver] = (dr->GetMetadataItem(GDAL_DCAP_CREATECOPY) != NULL);
+    iscreate[idriver] = (dr->GetMetadataItem(GDAL_DCAP_CREATE) != NULL);
+    isvirt[idriver] = (dr->GetMetadataItem(GDAL_DCAP_VIRTUALIO) != NULL);
   }
   Rcpp::List out = Rcpp::List::create(Rcpp::Named("driver") = sname,
                                       Rcpp::Named("name") = lname,
@@ -193,7 +192,7 @@ inline List gdal_list_drivers()
 // (between  names[i] = poFieldDefn->GetNameRef();
 // and out.attr("names") = names;
 // and remove the GetGeomFieldCount from the sum of n fields
-inline Rcpp::List allocate_fields_list(OGRFeatureDefn *poFDefn, int n_features, bool int64_as_string,
+inline Rcpp::List allocate_fields_list(OGRFeatureDefn *poFDefn, R_xlen_t n_features, bool int64_as_string,
                                        Rcpp::CharacterVector fid_column) {
   
   if (fid_column.size() > 1)
@@ -201,7 +200,7 @@ inline Rcpp::List allocate_fields_list(OGRFeatureDefn *poFDefn, int n_features,
   
   // modified MDS
   //int n = poFDefn->GetFieldCount() + poFDefn->GetGeomFieldCount() + fid_column.size();
-  int n = poFDefn->GetFieldCount() + fid_column.size();
+  int n = poFDefn->GetFieldCount() + (int)fid_column.size();
   
   Rcpp::List out(n);
   Rcpp::CharacterVector names(n);
@@ -223,8 +222,8 @@ inline Rcpp::List allocate_fields_list(OGRFeatureDefn *poFDefn, int n_features,
     case OFTDateTime: {
       Rcpp::NumericVector ret(n_features);
       Rcpp::CharacterVector cls(2);
-      cls(0) = "POSIXct";
-      cls(1) = "POSIXt";
+      cls[0] = "POSIXct";
+      cls[1] = "POSIXt";
       ret.attr("class") = cls;
       out[i] = ret;
     } break;
@@ -281,7 +280,7 @@ inline List gdal_read_fields(CharacterVector dsn,
   
   //double  nFeature = force_layer_feature_count(poLayer);
   // trying to fix SQL problem 2020-10-05
-  double nFeature = poLayer->GetFeatureCount();
+  R_xlen_t nFeature = poLayer->GetFeatureCount();
   if (nFeature < 1) {
     //Rprintf("force count\n");
     nFeature = force_layer_feature_count(poLayer);
@@ -312,7 +311,7 @@ inline List gdal_read_fields(CharacterVector dsn,
   
   OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
   bool int64_as_string = false;
-  List out = allocate_fields_list(poFDefn, nFeature, int64_as_string, fid_column_name);
+  List out = allocate_fields_list(poFDefn, (int)nFeature, int64_as_string, fid_column_name);
   int iFeature = 0;  // always increment iFeature, it is position through the loop
   int lFeature = 0; // keep a count of the features we actually send out
   while((poFeature = poLayer->GetNextFeature()) != NULL)
@@ -385,7 +384,7 @@ inline List gdal_read_fields(CharacterVector dsn,
 }
 
 
-inline DoubleVector gdal_feature_count(CharacterVector dsn,
+ inline NumericVector gdal_feature_count(CharacterVector dsn,
                                        IntegerVector layer, CharacterVector sql, NumericVector ex) {
   GDALDataset       *poDS;
   poDS = (GDALDataset*) GDALOpenEx(dsn[0], GDAL_OF_VECTOR, NULL, NULL, NULL );
@@ -399,7 +398,7 @@ inline DoubleVector gdal_feature_count(CharacterVector dsn,
   poLayer->ResetReading();
   //  double nFeature = force_layer_feature_count(poLayer);
   // trying to fix SQL problem 2020-10-05
-  double nFeature = poLayer->GetFeatureCount();
+  R_xlen_t nFeature = poLayer->GetFeatureCount();
   if (nFeature < 1) {
     //Rprintf("force count\n");
     nFeature = force_layer_feature_count(poLayer);
@@ -412,8 +411,8 @@ inline DoubleVector gdal_feature_count(CharacterVector dsn,
   }
   GDALClose( poDS );
   
-  DoubleVector out(1);
-  out[0] = nFeature;
+  NumericVector out(1);
+  out[0] = static_cast<double>(nFeature);
   return(out);
 }
 
@@ -483,7 +482,7 @@ inline List gdal_read_geometry(CharacterVector dsn,
   int lFeature = 0;
   //int   nFeature = force_layer_feature_count(poLayer);
   // trying to fix SQL problem 2020-10-05
-  double nFeature = poLayer->GetFeatureCount();
+  R_xlen_t nFeature = poLayer->GetFeatureCount();
   if (nFeature < 1) {
     //Rprintf("force count\n");
     nFeature = force_layer_feature_count(poLayer);
@@ -657,7 +656,7 @@ inline List gdal_read_names(CharacterVector dsn,
   int lFeature = 0;
   // double   nFeature = force_layer_feature_count(poLayer);
   // trying to fix SQL problem 2020-10-05
-  double nFeature = poLayer->GetFeatureCount();
+  R_xlen_t nFeature = poLayer->GetFeatureCount();
   if (nFeature < 1) {
     //Rprintf("force count\n");
     nFeature = force_layer_feature_count(poLayer);
